import iris

"""
scripts:
    -  /esarchive/scratch/jcos/es-esmvaltool/common-tools/projections/diagnostics/scripts/plot_pattern_correlation_maps.py
        - plot_2methods_regional_timeseries -> S6 & 7, remove RPSS boes and so on
    - /esarchive/scratch/jcos/es-esmvaltool/common-tools/projections/diagnostics/shared_constraints_diagnostics/member_selection_heatmap.py
        - heatmap S2 (inplace) add redbox, subtract first decade
    - /esarchive/scratch/jcos/es-esmvaltool/common-tools/projections/diagnostics/scripts/plot_relres.py
        - S4 & 5 (inplace?)
    S3 (done)
    S1 probably done in inkscape

"""

import matplotlib.pyplot as plt
import glob
import os
import warnings
import yaml
import json
import logging
import numpy as np
from dask import array as da
import esmvaltool.diag_scripts.shared as e
from esmvaltool.diag_scripts.shared import names, extract_variables, io
from datetime import datetime
import sys
import matplotlib.patches as patches

current = os.path.dirname(os.path.realpath(__file__))
parent_directory = os.path.dirname(current)
sys.path.append(parent_directory)

logger = logging.getLogger(__name__)
FORMAT = "%(asctime)-15s %(message)s"
logging.basicConfig(format=FORMAT)


startTime = datetime.now()
warnings.filterwarnings("ignore")
config_loops_file = "/esarchive/scratch/jcos/es-esmvaltool/common-tools/projections/diagnostics/config_loops.yml"
with open(config_loops_file, "r") as f:
    config_loops = yaml.full_load(f)


class IntermediaryPlots(object):
    def __init__(self, config, crit, cstr_yavg, crit_collect, i, N):
        self.cfg = config
        self.cfg["earl_anomaly"] = False
        self.cfg["selection_criteria"] = crit
        self.cfg["collect_criteria"] = crit_collect
        self.cfg["cstr_yavg"] = cstr_yavg
        self.cfg["subsample_size"] = N
        self.setup_dirs()
        if i != 0:
            keys = list(self.cfg["regions"].keys())
            [self.cfg["regions"].pop(key, None) for key in keys if "corr" not in key]

    def setup_dirs(self):
        criteria_label = self.cfg["selection_criteria"]
        year_averaging = self.cfg["cstr_yavg"]
        self.cfg["out_plot_dir"] = os.path.join(self.cfg[names.PLOT_DIR],f"{criteria_label}", year_averaging)
        if not os.path.isdir(self.cfg["out_plot_dir"]):
            os.makedirs(self.cfg["out_plot_dir"])
        self.cfg["out_work_dir"] = os.path.join(self.cfg[names.WORK_DIR],f"{criteria_label}", year_averaging)
        if not os.path.isdir(self.cfg["out_work_dir"]):
            os.makedirs(self.cfg["out_work_dir"])
        
        if "_eval-" in self.cfg["selection_criteria"]:
            if self.cfg["selection_criteria"].endswith("_X"):
                attributes = self.cfg["selection_criteria"].split("_")
                criteria_label = ""
                for attr in attributes:
                    if "eval-" not in attr:
                        criteria_label += f"{attr}_"
                criteria_label = criteria_label[:-1]
            else:
                criteria_label = self.cfg["selection_criteria"].split("_eval-")[0]

        new_idata = []
        for path in self.cfg["input_files"]:
            new_path = path
            if not path.endswith('metadata.yml'):
                new_path = os.path.join(path, criteria_label, year_averaging)
            new_idata.append(new_path)
        self.cfg["input_files"] = new_idata
        print(self.cfg["input_files"])

    
    def main(self):
        self.plot_heatmap()
        # self.plot_correlation_timeseries()

    def plot_heatmap(self, ):
        reverse = True if "RMSE" not in self.cfg["selection_criteria"] else False
        var = self.cfg['input_files'][0].split("/")[-3].split("_")[0]
        recipe_name = self.cfg['input_files'][0].split("/")[-6]
        if "observation" in self.cfg["recipe"]:
            method = "OBS"
        elif "decadal" in self.cfg["recipe"]:
            method = "DP"

        for region in self.cfg["regions"]:
            # if region != "NAtl":
            if region != "Glob":
                continue
            if "corr" in region:
                region = self.cfg["regions"][region].replace("CSTR-YRS",f"{self.cfg['cstr_yavg']}").replace("CRIT-CLCT",f"{self.cfg['collect_criteria']}")
                region = region.split("OBS_")[-1].split(".nc")[0]
            m_files = io.get_all_ancestor_files(self.cfg, f"{region}_*.json")
            m_files.sort()
            if len(m_files) == 0:
                continue
            formatter = {"RMSE": "RMSE", "PC": "Uncentered pat. corr.", "PCiris": "Centered pat. corr."}
            N=self.cfg["subsample_size"]
            corrs_array = []
            sds = []
            for i,m_sd in enumerate(m_files):
                logger.info(f"loading json correlation file: {m_sd}")
                with open(m_sd, "r") as f:
                    corr = json.load(f)
                    corrs = list(corr.values())
                corr = self.purge_dataset("CESM2", corr)
                if i == 0:
                    selection = np.zeros((len(m_files),len(list(corr.keys()))))
                    members = list(corr.keys())
                    members = sorted([m for m in corr if not m.startswith("OBS")])
                selected = [k for k, v in sorted(corr.items(), key=lambda item: item[1], reverse=reverse)][:N]
                seldict = {m: 0 for m in members}
                for d in seldict:
                    if d in selected:
                        seldict[d] = 1
                selection[i] = list(seldict.values())
              
                corrs.sort(reverse=reverse)
                corrs_array.append(corrs)
                sds.append(int(m_sd.split(".json")[0].split("_")[-1]))

            



            canesm = [i for i, m in enumerate(members) if "CanESM5" in m]
            members = [m.replace("_", " ") for m in members]
            dates = np.arange(1961, 2001)
            fig, ax = plt.subplots(figsize=(12,28/210*len(members)), dpi=150)
            fill = plt.pcolormesh(dates[9:], members, selection.transpose()[:,9:], cmap=plt.cm.Greys, vmin=0, vmax=1, edgecolors="black")
            ymin, ymax = plt.ylim()
            # plt.vlines(1969.5, ymin, ymax, color = "red", linewidth=2)
            # plt.colorbar(fill, location="right", orientation="vertical", shrink=0.3, cmap=plt.cm.Greys, label="Weight")
            plt.title(f"Selection of the Best{self.cfg['subsample_size']} through\n"
                       f"{method}-sel. {self.cfg['cstr_yavg']} {formatter[self.cfg['selection_criteria']]}\n"
                       f"in region: {region.split('_signi')[0]}\n", fontsize="x-large", pad=-10)
            plt.tight_layout()
            minimal_region = region
            if "corr" in minimal_region:
                minimal_region=minimal_region.partition(".")[-1].rpartition("_signi_corr-mask")[0]
            if "decadal" in self.cfg["out_plot_dir"]:
                method = "DP"
                rect = patches.Rectangle((1969.6, canesm[0]+ymin), 30.8, canesm[-1]-canesm[0]-ymin*2, linewidth=3, edgecolor='r', facecolor='none')
                ax.add_patch(rect)
                ax.text(-0.2, 1.02, "A)", transform=ax.transAxes, fontsize="xx-large")
            else:
                method = "OBS"
                ax.text(-0.2, 1.02, "B)", transform=ax.transAxes, fontsize="xx-large")
            plt.savefig(os.path.join("/esarchive/scratch/jcos/constraints_paper_figures_data_scritps/plots", f"{method}_selection_heatmap_best{self.cfg['subsample_size']}_{minimal_region}.png"))
            plt.savefig(os.path.join("/esarchive/scratch/jcos/constraints_paper_figures_data_scritps/plots", f"{method}_selection_heatmap_best{self.cfg['subsample_size']}_{minimal_region}.pdf"), format="pdf", bbox_inches='tight')

    def purge_dataset(self, datasets, dictionary):
        """
        Purge a list of datasets from the dictionary
        datasets: List of strings
        dictionary: Keys are datasets or aliases
        """
        if not isinstance(datasets, list):
            datasets = [datasets]
        keys_list = list(dictionary.keys())
        purgeable_keys = []
        for dataset in datasets:
            full_name_dataset_keys=[k for k in keys_list if dataset == k]
            part_name_dataset_keys=[k for k in keys_list if dataset in k.split("_")]
            dataset_keys=full_name_dataset_keys+part_name_dataset_keys
            purgeable_keys += dataset_keys
        return {k: v for k, v in dictionary.items() if k not in purgeable_keys}
    
def main():
    print("----------- MAIN ----------")
    for crit in config_loops["criteria_cstr"]:
        for cstr_yavg in config_loops["year_averages"]:
            for N in config_loops["subsample_size"]:
                for i, crit_collect in enumerate(config_loops["criteria_collect"]):
                    with e.run_diagnostic() as config:
                        IntermediaryPlots(config, crit, cstr_yavg, crit_collect, i, N).main()


if __name__ == "__main__":
    main()
    print(f" TIME SPENT: {datetime.now() - startTime}")
